Attribute VB_Name = "modCompiler"
Enum EAsmType
  a_AS65
  a_DASM      ' not implemented
  a_TASM
End Enum
Public iAsm As EAsmType

Public sResFile As String ' Result File

Public Function Mnemonics() As String
  Dim s As String
  '
  s = "LDA/LDX/LDY/STA/STX/STY/"
  s = s + "CMP/CPX/CPY/BIT/INC/INX/INY/DEC/DEX/DEY/ADC/SBC/"
  s = s + "AND/ORA/EOR/"
  s = s + "ASL/LSR/ROR/ROL/"
  s = s + "PHA/PLA/PHP/PLP/"
  s = s + "JMP/JSR/RTS/RTI/"
  s = s + "BEQ/BNE/BPL/BMI/BCC/BCS/BVC/BVS/"
  s = s + "TAX/TXA/TAY/TYA/TSX/TXS/"
  s = s + "SEI/CLI/SEC/CLC/SED/CLD/CLV/"
  s = s + "NOP/BRK/"
  s = Replace(s, "/", Chr$(10))
  '
  Mnemonics = s
End Function

Public Function Commands(Optional assemblerType As EAsmType = a_AS65) As String
  Dim s As String
  '
  Select Case assemblerType
  Case a_AS65
    s = s & "ALIGN/"
    s = s & "BSS/"
    s = s & "CMAP/"
    s = s & "CODE/"
    s = s & "DATA/"
    s = s & "DB/"
    s = s & "DS/"
    s = s & "DW/"
    s = s & "ELSE/"
    s = s & "END/"
    s = s & "END STRUCT/"
    s = s & "ENDIF/"
    s = s & "ENDM/"
    s = s & "EQU/"
    s = s & "FCB/"
    s = s & "FCC/"
    s = s & "FCW/"
    s = s & "FDB/"
    s = s & "IF/"
    s = s & "INCLUDE/"
    s = s & "LIST/"
    s = s & "MACRO/"
    s = s & "NOLIST/"
    s = s & "NOOPT/"
    s = s & "NOP/"
    s = s & "OPT/"
    s = s & "ORG/"
    s = s & "PAGE/"
    s = s & "RMB/"
    s = s & "SET/"
    s = s & "STRUCT/"
    s = s & "TITLE/"
  Case a_TASM
    s = s & ".ADDISNTR/"
    s = s & ".AVSYM/"
    s = s & ".BLOCK/"
    s = s & ".BSEG/"
    s = s & ".CSEG/"
    s = s & ".DSEG/"
    s = s & ".NSEG/"
    s = s & ".XSEG/"
    s = s & ".BYTE/"
    s = s & ".CHK/"
    s = s & ".CODES/"
    s = s & ".NOCODES/"
    s = s & ".DB/"
    s = s & ".DW/"
    s = s & "#DEFINE/"
    s = s & "#DEFCONT/"
    s = s & ".EJECT/"
    s = s & "#ELSE/"
    s = s & ".END/"
    s = s & "#ENDIF/"
    s = s & ".EQU/"
    s = s & ".EXPORT/"
    s = s & "#IFDEF/"
    s = s & "#IFNDEF/"
    s = s & "#IF/"
    s = s & "#INCLUDE/"
    s = s & ".LIST/"
    s = s & ".NOLIST/"
    s = s & ".LOCALLABELCHAR/"
    s = s & ".LSFIRST/"
    s = s & ".MSFIRST/"
    s = s & ".ORG/"
    s = s & ".PAGE/"
    s = s & ".NOPAGE/"
    s = s & ".SET/"
    s = s & ".SYM/"
    s = s & ".TEXT/"
    s = s & ".TITLE/"
    s = s & ".WORD/"
  Case a_DASM
    s = ""
  End Select
  '
  Commands = s
End Function

Private Function Operators(Optional assemblerType As EAsmType = a_AS65) As String
  Dim s As String
  '
  Select Case assemblerType
  Case a_AS65
    s = "+,-,*,/,&,>>,<<,!,~,<,>,<=,>=,=,!=,^,|,&&,||,hi,lo"
  Case a_TASM
    s = "+,-,*,/,&,>>,<<,!,~,<,>,<=,>=,=,==,!=,^,|"
  Case a_DASM
    s = ""
  End Select
  s = Replace(s, ",", Chr$(10))
  '
  Operators = s
End Function

Private Function ScopeKeywords1(Optional assemblerType As EAsmType = a_AS65) As String
  Dim s As String
  '
  Select Case assemblerType
  Case a_AS65
    s = "{,MACRO,STRUCT,IF"
  Case a_TASM
    s = ""
  Case a_DASM
    s = ""
  End Select
  s = Replace(s, ",", Chr$(10))
  '
  ScopeKeywords1 = s
End Function

Private Function ScopeKeywords2(Optional assemblerType As EAsmType = a_AS65) As String
  Dim s As String
  '
  Select Case assemblerType
  Case a_AS65
    s = "},ENDM,END STRUCT,ENDIF"
  Case a_TASM
    s = ""
  Case a_DASM
    s = ""
  End Select
  s = Replace(s, ",", Chr$(10))
  '
  ScopeKeywords2 = s
End Function

'
' Register the Plus/4 assemblers as a custom languages for syntax highlighting
'
Public Sub RegisterCustomLanguage(ByVal Control As CodeMaxCtl.ICodeMax, Optional assemblerType As EAsmType = a_AS65)
  Dim lang As CodeMaxCtl.Language
  '
  Set lang = New CodeMaxCtl.Language
  lang.CaseSensitive = True
  '
  Dim s As String
  lang.Keywords = Mnemonics
  '
  lang.Style = cmLangStyleProcedural
  lang.Operators = Operators(assemblerType)
  lang.SingleLineComments = ";"
  lang.MultiLineComments1 = ""
  lang.MultiLineComments2 = ""
  lang.ScopeKeywords1 = ScopeKeywords1(assemblerType)
  lang.ScopeKeywords2 = ScopeKeywords2(assemblerType)
  lang.StringDelims = Chr$(34) + Chr$(10) + "'"
  lang.EscapeChar = ""
  lang.TerminatorChar = ""
  '
  Dim globals As CodeMaxCtl.globals
  Set globals = New CodeMaxCtl.globals
  '
  Select Case assemblerType
  Case a_AS65
    Call globals.RegisterLanguage("AS65 Plus4 Assembler", lang)
    Control.Language = "AS65 Plus4 Assembler"
  Case a_TASM
    Call globals.RegisterLanguage("TASM Plus4 Assembler", lang)
    Control.Language = "TASM Plus4 Assembler"
  Case a_DASM
    Call globals.RegisterLanguage("DASM Plus4 Assembler", lang)
    Control.Language = "DASM Plus4 Assembler"
  End Select
End Sub

Private Function ToPETSCII(s As String) As String
  Dim i As Integer
  Dim c As Integer
  Dim s2 As String
  '
  Select Case iAsm
    Case a_AS65
      s2 = Chr$(9) & "DB "
    Case a_DASM
      s2 = Chr$(9) & "DC.B "
    Case a_TASM
      s2 = Chr$(9) & ".DB "
  End Select
  '
  For i = 1 To Len(s)
    '
    c = Asc(Mid$(s, i, 1))
    Select Case c
      '
      Case 91, 93
        c = c - 64
        '
      Case 97 To 122
        c = c - 96
        '
    End Select
    '
    s2 = s2 & "$" & Hex$(c)
    '
    If i < Len(s) Then
      s2 = s2 & ","
    End If
  Next i
  '
  ToPETSCII = s2
End Function

Public Sub ConvertToPETSCII(ByRef cm As CodeMaxCtl.CodeMax)
  Dim X As New Range
  Dim p As New Position
  Dim i As Long
  Dim j As Long
  Dim s As String
  '
  If iCurrent >= 0 Then
    With cm
      Set X = .GetSel(True)
      j = X.StartLineNo
      For i = X.StartLineNo To X.EndLineNo
        s = .GetLine(i)
        s = ToPETSCII(Mid$(s, 2))
        '.DeleteLine (i)
        '.InsertLine i, s
        j = j + 1
        .InsertLine j, s
      Next i
    End With
  End If
  '
End Sub

Private Function IsSineDescriptor(s As String) As Boolean
  Dim i As Long
  Dim t As String
  If s = "" Then IsSineDescriptor = False: Exit Function
  For i = 1 To Len(s)
    If Mid(s, i, 1) Like "[! ,0-9]" Then IsSineDescriptor = False: Exit Function
  Next i
  IsSineDescriptor = True
End Function

Public Sub GenerateSineTable(ByRef cm As CodeMaxCtl.CodeMax)
  On Error GoTo GError
  '
  Dim X As New Range
  Dim p As New Position
  Dim i As Long
  Dim j As Long
  Dim s As String
  Dim Values
  '
  Dim iWaveLength As Integer
  Dim dAmplitude As Double
  Dim dAmpShift As Double
  Dim iWaveShift As Integer
  Dim dRepeat As Double
  '
  If iCurrent >= 0 Then
    With cm
      Set X = .GetSel(True)
      j = X.StartLineNo
      For i = X.StartLineNo To X.EndLineNo
        s = XTrim$(.GetLine(i))
        '
        If Left$(s, 1) = ";" Then s = XTrim$(Mid$(s, 2)) ' remove comment mark
        '
        If Not IsSineDescriptor(s) Then
          MsgBox "The current editor line is ambiguous." & vbCrLf & vbCrLf & _
            "To generate a sine table, create a comment line like this: " & vbCrLf & _
            "; <wavelength>,<amplitude>,<amplitude shift>,<wave shift>,<repeat>" & vbCrLf & vbCrLf & _
            "Only the first parameter is mandatory, the rest are optional." & vbCrLf & _
            "Amplitude and shift can be any number, wavelength and shift must be integers." & vbCrLf & _
            "Example: ; 256,27.5", vbInformation
        Else
          '
          Values = Split(s, ",")
          '
          iWaveLength = CInt(Val(Values(0)))
          dAmplitude = 64
          dAmpShift = 0
          iWaveShift = 0
          If UBound(Values) > 0 Then
            dAmplitude = Val(Values(1))
          End If
          If UBound(Values) > 1 Then
            dAmpShift = Val(Values(2))
          End If
          If UBound(Values) > 2 Then
            iWaveShift = CInt(Val(Values(3)))
          End If
          If UBound(Values) > 3 Then
            dRepeat = Val(Values(4))
          End If
          '
          s = GenerateSine(Val(Values(0)), dAmplitude, dAmpShift, iWaveShift, dRepeat)
          '
          p.ColNo = 0
          p.LineNo = j + 1
          .InsertText s, p
          '
          cm.Container.Status "Generated sine table: " & iWaveLength & ", " & dAmplitude & ", " & dAmpShift & "," & iWaveShift
        End If
      Next i
    End With
  End If
  '
  Exit Sub
GError:
  MsgBox Err.Description, vbExclamation, Err.Number
End Sub

Private Function GenerateSine(ByVal Wavelength As Integer, ByVal Amplitude As Double, ByVal AmpShift As Double, ByVal WaveShift As Integer, ByVal dRepeat As Double) As String
  On Error GoTo GError
  '
  Dim s As String
  Dim i As Integer
  Dim Value As Double
  Dim RoundedValue As Integer
  Dim Pi As Double
  '
  Pi = 3.14159265358979
  '
  If dRepeat = 0 Then dRepeat = 1
  '
  For i = 0 To (Wavelength * dRepeat) - 1
    Value = AmpShift + Amplitude + Amplitude * Sin((i + WaveShift) * Pi / (Wavelength / 2))
    RoundedValue = Int(Value + 0.5)
    'Debug.Print RoundedValue
    If (i Mod Wavelength) Mod 16 = 0 Then
      Select Case iAsm
      Case a_AS65
        s = s & vbTab & "DB "
      Case a_DASM
        s = s & vbTab & "DC.B "
      Case a_TASM
        s = s & vbTab & ".DB "
      End Select
    Else
      s = s & ","
    End If
    s = s & "$" + Right$("00" + Hex$(RoundedValue), 2)
    If ((i + 1) Mod Wavelength) Mod 16 = 0 Then
      s = s & vbCrLf
    End If
  Next i
  GenerateSine = s
  '
  Exit Function
GError:
  MsgBox Err.Description, vbExclamation, Err.Number
End Function
